/*
 * file bomb.h - bombs and explosions
 *
 * $Id: bomb.h,v 1.11 2001/06/27 05:55:37 xblast Exp $
 *
 * Program XBLAST 
 * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2; or (at your option)
 * any later version
 *
 * This program is distributed in the hope that it will be entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef XBLAST_BOMB_H
#define XBLAST_BOMB_H

#include "player.h"

/*
 * typedef definitions
 */

/* bomb sprites */
typedef enum {
  BB_NORMAL=0,
  BB_MINI
} BMBombSprite;
/* bomb types */
typedef enum {
  BMTevil = -3,     /* hidden bomb */
  BMTspecial,       /* bomb dropped with special key */
  BMTdefault,       /* bomb dropped with bomb key */
  BMTnormal,        /* standard bomb */
  BMTnapalm,        /* napalm bomb, area effect*/
  BMTblastnow,      /* instant explosion */
  BMTclose,         /* bombs used for "nasty walls" */
  BMTfirecracker,   /* firecracker */
  BMTfirecracker2,  /* bombs created by firecracker explosion */
  BMTconstruction,  /* create a blastable block on explosion */
  BMTthreebombs,    /* three bombs in a row */
  BMTgrenade,       /* grenade, another area effect */
  BMTtrianglebombs, /* three bombs forming a triangle */
  BMTdestruction,   /* this bombs destroys walls */
  BMTfungus,        /* this bomb mulitplies */
  BMTrenovation,    /* this bomb move blocks */
  BMTpyro,          /* pyro bomb */ 
  BMTpyro2,         /* bombs created by pyro explosion */ 
  BMTrandom,        /* random bomb dtermined after drop */
  BMTshort,         /* bomb with very short fuse */
  NUM_BMT
} BMBombType;

/*
 * prototypes
 */
extern XBBool ConfigLevelBombs (const DBSection *section);
extern void DeleteAllExplosions (void);
extern void DoBombs (void);
extern int IgnitePlayersBombs (BMPlayer *ps);
extern int IgniteAllBombs (BMPlayer *ps);
extern void IgniteBombs (void);
extern void DoExplosions (void);
extern XBBool NewPlayerBomb (BMPlayer *ps, BMBombType type);
extern XBBool NewEvilBomb (int x, int y);
extern void StunPlayers (BMPlayer *ps, int numPlayer);
extern XBBool CheckBomb (int x, int y);
extern int NumberOfExplosions (void);
extern void DeleteBombAt (int x, int y);
extern void MoveBomb (int x, int y, int dir);
extern int CheckDistribExpl (unsigned *distExtra, int freeBlocks);
extern void HauntKick (int prob);
extern void DoAir (BMPlayer *ps);
extern void DoNastyWalls (int gameTime);

#endif
/*
 * end of file bomb.h
 */






























