/*
 * cfg_game.h - game configuration data
 *
 * $Id: cfg_game.h,v 1.28 2001/08/16 20:04:17 xblast Exp $
 *
 * Program XBLAST 
 * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2; or (at your option)
 * any later version
 *
 * This program is distributed in the hope that it will be entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef XBLAST_CFG_GAME_H
#define XBLAST_CFG_GAME_H

#include "cfg_main.h"
#include "color.h"
#include "event.h"
#include "net_tele.h"

/* player controls */
typedef enum {
  XBPC_None,
  XBPC_RightKeyboard,
  XBPC_LeftKeyboard,
  XBPC_Joystick1,
  XBPC_Joystick2,
  XBPC_Joystick3,
  XBPC_Joystick4,
  /* no new elements after this line */
  XBPC_NUM
} XBPlayerControl;

/*player hosts */
typedef enum {
  XBPH_None,
  XBPH_Demo,
  XBPH_Local,
  XBPH_Server,
  XBPH_Client1,
  XBPH_Client2,
  XBPH_Client3,
  XBPH_Client4,
  XBPH_Client5,
  /* no new elements after this line */
  XBPH_NUM
} XBPlayerHost;

/*player teams */
typedef enum {
  XBPT_None,
  XBPT_Red,
  XBPT_Green,
  XBPT_Blue,
  /* no new elements after this line */
  XBPT_NUM
} XBPlayerTeam;

/* game player config  */
typedef struct {
  int             num;
  XBAtom          player[MAX_PLAYER];
  XBPlayerControl control[MAX_PLAYER];
  XBPlayerHost    host[MAX_PLAYER];
  XBPlayerTeam    team[MAX_PLAYER];
  XBColor         teamColor[MAX_PLAYER];
} CFGGamePlayers;

/* team modes */
typedef enum {
  XBTM_None,        /* normal game */
  XBTM_RandomTwo,   /* two teams, assignment is random (4 or 6 player) */
  XBTM_RandomThree, /* three teams, assignment is random (6 players only) */
  XBTM_Hunt,        /* all on one (3 to 6 players) */
  /* no new elements after this line */
  XBTM_NUM
} XBTeamMode;

/* game parameters */
typedef struct {
  int        numLives;
  int        numWins;
  int        frameRate;
  XBBool     allLevels;
  XBBool     randomLevels;
  XBBool     randomPlayers;
  XBBool     recordDemo;
  XBTeamMode teamMode;
} CFGGameSetup;

/* connection to host */
typedef struct {
  const char *name;
  int         port;
  XBBool      fixedUdpPort;
  XBBool      browseLan;
  XBBool      allowNat;
  const char *game;
} CFGGameHost;

/* all in one */
typedef struct {
  CFGGameSetup   setup;
  CFGGamePlayers players;
  CFGGameHost    host;
} CFGGame;

/* 
 * global prototypes
 */
extern void LoadGameConfig (void);
extern void SaveGameConfig (void);
extern void FinishGameConfig (void);

extern void StoreGame        (CFGType, XBAtom atom, const CFGGame *);
extern void StoreGameHost    (CFGType, XBAtom atom, const CFGGameHost *);
extern void StoreGameSetup   (CFGType, XBAtom atom, const CFGGameSetup *);
extern void StoreGamePlayers (CFGType, XBAtom atom, const CFGGamePlayers *);

extern XBBool RetrieveGame        (CFGType, XBAtom atom, CFGGame *);
extern XBBool RetrieveGamePlayers (CFGType, XBAtom atom, CFGGamePlayers *);
extern XBBool RetrieveGameSetup   (CFGType, XBAtom atom, CFGGameSetup   *);
extern XBBool RetrieveGameHost    (CFGType, XBAtom atom, CFGGameHost  *);

extern XBBool SendGameConfig   (CFGType, XBSndQueue *sndQueue, XBTeleCOT cot, XBTeleIOB iob, XBAtom atom);
extern void AddToGameConfig    (CFGType, XBAtom atom, const char *text);
extern void DeleteGameConfig   (CFGType, XBAtom);
extern const char *GetHostName (CFGType, XBAtom);

#endif
/*
 * end of file cfg_game.h
 */
