/*
 * func.c - function pointers for special extras
 *
 * $Id: func.c,v 1.17 2001/02/22 18:44:08 xblast Exp $
 *
 * Program XBLAST 
 * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2; or (at your option)
 * any later version
 *
 * This program is distributed in the hope that it will be entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "func.h"

#include "atom.h"
#include "bomb.h"
#include "geom.h"
#include "info.h"
#include "snd.h"
#include "random.h"

/*
 * global variables
 */
void (*specialExtraFunc) (BMPlayer *);
void (*specialKeyFunc) (BMPlayer *);

/*
 * local types
 */
typedef void (*SpecialExtraFunc) (BMPlayer *);
typedef void (*SpecialKeyFunc) (BMPlayer *);

/* 
 * void functions 
 */
static void 
SpecialExtraVoid (BMPlayer *ps)
{
}

static void 
SpecialKeyVoid (BMPlayer *ps)
{
}

/* 
 * Invincible 
 */
static void 
SpecialExtraInvincible (BMPlayer *ps)
{
  SND_Play (SND_INVINC, ps->x / (PIXW / MAX_SOUND_POSITION));
  ps->illtime = 0;
  ps->illness = ps->health;
  ps->invincible += EXTRA_INVINCIBLE;
} /* SpecialExtraInvincible */

/* 
 * Kicking 
 */
static void 
SpecialExtraKick (BMPlayer *ps)
{
  SND_Play (SND_NEWKICK, ps->x / (PIXW / MAX_SOUND_POSITION));
  ps->kick = XBTrue;
} /* SpecialExtraKick */

/* 
 * Remote Control 
 */
static void 
SpecialExtraRC (BMPlayer *ps)
{
  SND_Play (SND_NEWRC, ps->x / (PIXW / MAX_SOUND_POSITION));
  ps->remote_control = XBTrue;
} /* SpecialExtraRC */

static void 
SpecialKeyRC (BMPlayer *ps)
{
  if (ps->remote_control > 0) {
    if (IgnitePlayersBombs (ps)) {
      SND_Play (SND_SHOOT, ps->x / (PIXW / MAX_SOUND_POSITION));
    }
  }
} /* SpecialKeyRC */

/* 
 * Teleport 
 */
static void 
SpecialExtraTeleport (BMPlayer *ps)
{
  SND_Play (SND_NEWTELE, ps->x / (PIXW / MAX_SOUND_POSITION));
  if (ps->teleport == 0) {
    ps->teleport = 1;
  }
} /* SpecialExtraTeleport */

static void 
SpecialKeyTeleport (BMPlayer *ps)
{
  if (ps->teleport == 1) {
    ps->teleport = TELEPORT_TIME;
  }
} /* SpecialKeyTeleport */

/* 
 * Extra Ignite All 
 */
static void 
SpecialExtraIgniteAll (BMPlayer *ps)
{
  SND_Play (SND_BUTT, ps->x / (PIXW / MAX_SOUND_POSITION));
  IgniteAllBombs (ps);
} /* SpecialExtraIgniteAll */

static void 
SpecialKeySpecialBomb (BMPlayer *ps)
{
  DropBomb (ps, BMTspecial);
} /* SpecialKeySpecialBomb */

static void 
SpecialExtraSpecialBomb (BMPlayer *ps)
{
  SND_Play (SND_SPBOMB, ps->x / (PIXW / MAX_SOUND_POSITION));
  ps->special_bombs += 3;
} /* SpecialextraSpecialBomb */

/*
 * Junkie (Garth again)
 */
static void 
SpecialExtraJunkie (BMPlayer *ps)
{
  SND_Play (SND_INJ, ps->x / (PIXW / MAX_SOUND_POSITION));
  ps->junkie = MAX_JUNKIE_TIME;
} /* SpecialExtraJunkie */

/* 
 * Air Pump (Garth Denley) 
 */
static void 
SpecialExtraAir (BMPlayer *ps)
{
  SND_Play (SND_NEWPUMP, ps->x / (PIXW / MAX_SOUND_POSITION));
  ps->air_button = XBTrue;
} /* SpecialExtraAir */

static void 
SpecialKeyAir (BMPlayer *ps)
{
  if ((ps->air_button > 0) && (ps->lives > 0)) {
    SND_Play (SND_PUMP, ps->x / (PIXW / MAX_SOUND_POSITION));
    DoAir (ps);
  }
} /* SpecialKeyAir */

/*
 * poison extra
 */
static void
SpecialExtraPoison (BMPlayer *ps)
{
  if (! ps->invincible) {
    ps->dying = DEAD_TIME;
  }
} /* SpecialExtraPoison */

/*
 * spinner extra (long stunned )
 */
static void 
SpecialExtraLongStunned (BMPlayer *ps)
{
  SND_Play (SND_STUN, ps->x / (PIXW / MAX_SOUND_POSITION));
  ps->stunned = STUN_TIME*4;
} /* SpecialExtraLongStunned */

/*
 * speed extra
 */
static void
SpecialExtraSpeed (BMPlayer *ps)
{
  SND_Play (SND_FAST, ps->x / (PIXW / MAX_SOUND_POSITION));
  ps->illness = IllRun;
  ps->health  = IllRun;
  ps->illtime = 0;
} /* SpecialExtraSpeed */

/*
 * slow extra
 */
static void
SpecialExtraSlow (BMPlayer *ps)
{
  SND_Play (SND_SLOW, ps->x / (PIXW / MAX_SOUND_POSITION));
  ps->illness = IllSlow;
  ps->health  = IllSlow;
  ps->illtime = 0;
} /* SpecialExtraSlow */

/*
 * mayhem extra
 */
static void
SpecialExtraMayhem (BMPlayer *ps)
{
  SND_Play (SND_SLAY, ps->x / (PIXW / MAX_SOUND_POSITION));
  ps->kick    = XBTrue;
  ps->illness = IllRun;
  ps->health  = IllRun;
  ps->illtime = 0;
} /* SpecialExtraMayhem */

/*
 * holy grail extra
 */
static void
SpecialExtraHolyGrail (BMPlayer *ps)
{
  SND_Play (SND_HOLY, ps->x / (PIXW / MAX_SOUND_POSITION));
  ps->lives += KillOtherPlayers (ps->team);
  if (ps->lives > 9) {
    ps->lives = 9;
  }
  ps->invincible = NEW_INVINCIBLE;
  ps->dying      = DEAD_TIME;
} /* SpecialExtraHolyGrail */

/*
 * multiple extra 
 */
static void
SpecialExtraLife (BMPlayer *ps)
{
  SND_Play (SND_LIFE, ps->x / (PIXW / MAX_SOUND_POSITION));
  if (ps->lives < 9) {
    ps->lives ++;
  }
} /* SpecialExtraLife */


/*
 * multiple extra 
 */
static void
SpecialExtraMultiple (BMPlayer *ps)
{
  switch (GameRandomNumber (11)) {
  case 0:
  case 1:
    SpecialExtraSpeed (ps);
    break;
  case 2:
  case 3:
    SpecialExtraPoison (ps);
    break;
  case 4:
  case 5:
    SpecialExtraInvincible (ps);
    break;
  case 6:
  case 7:
    SpecialExtraLongStunned (ps);
    break;
  case 8:
  case 9:
    SpecialExtraAir (ps);
    break;
  case 10:
    SpecialExtraLife (ps);
    break;
  }
} /* SpecialExtraMultiple */

/*
 * cloakung device
 */
static void
SpecialExtraCloak (BMPlayer *ps)
{
  if (ps->cloaking < 0) {
    ps->cloaking *= -1;
  }
  ps->cloaking += 2*EXTRA_INVINCIBLE;
  SND_Play (SND_NEWCLOAK, ps->x / (PIXW / MAX_SOUND_POSITION));
} /* SpecialExtraCloak */

static void
SpecialKeyCloak (BMPlayer *ps)
{
  ps->cloaking *= -1;
  if (ps->cloaking > 0) {
    SND_Play (SND_DECLOAK, ps->x / (PIXW / MAX_SOUND_POSITION));
  } else if (ps->cloaking < 0) {
    SND_Play (SND_ENCLOAK, ps->x / (PIXW / MAX_SOUND_POSITION));
  }
} /* SpecialKeyCloak */

/*
 * pow extra (stun others)
 */
static void 
SpecialExtraStunOthers (BMPlayer *ps)
{
  StunOtherPlayers (ps->team, STUN_TIME*4);
}

/*
 * morphing extra (player becomes a bomb)
 */
static void
SpecialExtraMorph (BMPlayer *ps)
{
  ps->num_morph ++;
}

static void
SpecialKeyMorph (BMPlayer *ps)
{
  if ( (0 < ps->num_morph) &&
       (0 < ps->bombs) &&
       (! ps->morphed) ) {
#ifdef DEBUG
    fprintf (stderr, "Player %d morphs", ps->id);
#endif
    ps->morphed = 1;
    DropBomb (ps, BMTdefault);
  }
} /* SpecialKeyMorph */


/*
 * conversion tables
 */
/* extra pickup functions */
static DBToData extraTable[] = {
  { "air",         (void *) SpecialExtraAir },         
  { "cloak",       (void *) SpecialExtraCloak },       
  { "holyGrail",   (void *) SpecialExtraHolyGrail },   
  { "igniteAll",   (void *) SpecialExtraIgniteAll },   
  { "invincible",  (void *) SpecialExtraInvincible }, 
  { "junkie",      (void *) SpecialExtraJunkie },      
  { "kick",        (void *) SpecialExtraKick },        
  { "life",        (void *) SpecialExtraLife },        
  { "longStunned", (void *) SpecialExtraLongStunned }, 
  { "mayhem",      (void *) SpecialExtraMayhem },      
  { "morph",       (void *) SpecialExtraMorph },       
  { "multiple",    (void *) SpecialExtraMultiple },    
  { "poison",      (void *) SpecialExtraPoison },      
  { "rc",          (void *) SpecialExtraRC },          
  { "slow",        (void *) SpecialExtraSlow },        
  { "specialBomb", (void *) SpecialExtraSpecialBomb }, 
  { "speed",       (void *) SpecialExtraSpeed },       
  { "stunOthers" , (void *) SpecialExtraStunOthers },  
  { "teleport",    (void *) SpecialExtraTeleport },    
  { NULL, NULL }
};
/* extra key functions */
static DBToData keyTable[] = {
  { "air",	     (void *) SpecialKeyAir },         
  { "cloak",	     (void *) SpecialKeyCloak },       
  { "morph",	     (void *) SpecialKeyMorph },       
  { "rc",            (void *) SpecialKeyRC },          
  { "specialBomb",   (void *) SpecialKeySpecialBomb }, 
  { "teleport",	     (void *) SpecialKeyTeleport },    
  { NULL, NULL }
};

/*
 * parse section [FUNC]
 */
XBBool
ConfigLevelFunc (const DBSection *section)
{
  void *ptr;

  /* special extra func */
  if (DB_ConvertEntryData (section, atomExtra, &ptr, extraTable) ) {
    assert (ptr !=  NULL);
    specialExtraFunc = (SpecialExtraFunc) ptr;
  } else {
    specialExtraFunc = SpecialExtraVoid;
  }
  /* special key func */
  if (DB_ConvertEntryData (section, atomKey, &ptr, keyTable) ) {
    assert (ptr !=  NULL);
    specialKeyFunc = (SpecialKeyFunc) ptr;
  } else {
    specialKeyFunc = SpecialKeyVoid;
  }
  /* set extra info */
  if (specialExtraFunc == SpecialExtraAir) {
    AddExtraInfo ("Airpump as an extra");
  } else if (specialExtraFunc == SpecialExtraCloak) {
    AddExtraInfo ("Cloak as an extra");
  } else if (specialExtraFunc == SpecialExtraHolyGrail) {
    AddExtraInfo ("The Holy Grail as an extra");
  } else if (specialExtraFunc == SpecialExtraIgniteAll) {
    AddExtraInfo ("Button as an extra");
  } else if (specialExtraFunc == SpecialExtraInvincible) {
    AddExtraInfo ("Invincibility as an extra");
  } else if (specialExtraFunc == SpecialExtraJunkie) {
    AddExtraInfo ("Junkie virus as an extra");
  } else if (specialExtraFunc == SpecialExtraKick) {
    AddExtraInfo ("Kick as an extra");
  } else if (specialExtraFunc == SpecialExtraLife) {
    AddExtraInfo ("Free life as an extra");
  } else if (specialExtraFunc == SpecialExtraLongStunned) {
    AddExtraInfo ("Spinner as an extra");
  } else if (specialExtraFunc == SpecialExtraMayhem) {
    AddExtraInfo ("Mayhem /Kick & Run) as an extra");
  } else if (specialExtraFunc == SpecialExtraMorph) {
    AddExtraInfo ("Morphing as an extra");
  } else if (specialExtraFunc == SpecialExtraMultiple) {
    AddExtraInfo ("Random special extra");
  } else if (specialExtraFunc == SpecialExtraPoison) {
    AddExtraInfo ("Poison as an extra");
  } else if (specialExtraFunc == SpecialExtraRC) {
    AddExtraInfo ("Remote control as an extra");
  } else if (specialExtraFunc == SpecialExtraSlow) {
    AddExtraInfo ("Slowdown as an extra");
  } else if (specialExtraFunc == SpecialExtraSpeed) {
    AddExtraInfo ("Speed as an extra");
  } else if (specialExtraFunc == SpecialExtraStunOthers) {
    AddExtraInfo ("Stunner as an extra");
  } else if (specialExtraFunc == SpecialExtraTeleport) {
    AddExtraInfo ("Teleporter as an extra");
  }
  return XBTrue;
} /* ConfigLevelFunc */

/*
 *
 */
XBBool
HasSpecialBombs (void)
{
  return (specialExtraFunc == SpecialExtraSpecialBomb);
} /* HasSpecialBombs */

/*
 * end of file func.c
 */

