/*
 * file menu_extras.c - user interface for extras like demo playback and statistics
 *
 * $Id: menu_extras.c,v 1.12 2001/07/01 16:04:09 xblast Exp $
 *
 * Program XBLAST 
 * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2; or (at your option)
 * any later version
 *
 * This program is distributed in the hope that it will be entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "menu_extras.h"

#include "menu.h"
#include "menu_game.h"
#include "menu_layout.h"
#include "mi_tool.h"

/*
 * local vriables
 */
static XBStatData *datStat   = NULL;
static size_t      numStat   = 0;
static ssize_t     firstStat = 0;
static ssize_t     lastStat  = 0;

static MIC_button  exitFunc   = NULL;
static MIC_button  selectFunc = NULL;

static CFGDemoEntry *demoList  = NULL;
static size_t        numDemos  = 0;
static ssize_t       firstDemo = 0;
static ssize_t       lastDemo  = 0;

/*
 * local prototype
 */
static XBBool CreateStatMenu (void *par);
static XBBool CreateDemoMenu (void *par);

/*
 * Forward in selection
 */
static XBBool
ButtonForward (void *par)
{
  if (lastStat < numStat) {
    firstStat = lastStat;
  }
  return CreateStatMenu (par);
} /* ButtonForward */

/*
 * Forward in selection
 */
static XBBool
ButtonBackward (void *par)
{
  firstStat -= 2*MAX_GSTAT_ROWS;
  if (firstStat < 0) {
    firstStat = 0;
  }
  return CreateStatMenu (par);
} /* ButtonForward */

/*
 * back to extras menu
 */
XBBool 
ButtonBack (void *par)
{
  MIC_button func = (MIC_button) par;

  assert (func != NULL);
  assert (datStat != NULL);
  /* --- */
  free (datStat);
  datStat    = 0;
  numStat    = 0;
  firstStat  = 0;
  lastStat   = 0;
  exitFunc   = NULL;
  selectFunc = NULL;
  /* */
  return (*func) (NULL);
} /* ButtonBack */

/*
 * 
 */
static XBBool
CreateStatMenu (void *par)
{
  size_t  i, j;

  MenuClear ();
  /* Title */
  MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, "Player Statistics");
  /* list buttons */
  MenuSetActive (MenuAddStatHeader (GSTAT_LEFT, GSTAT_TOP, GSTAT_WIDTH, par), XBFalse);
  for (i = 0, j = firstStat; j < numStat && i < MAX_GSTAT_ROWS; i ++, j ++) {
    MenuSetActive (MenuAddStatEntry (GSTAT_LEFT, GSTAT_ROW (i), GSTAT_WIDTH, datStat + j, selectFunc, datStat + j),
		   (selectFunc != NULL) );
  }
  lastStat = j;
  /* Buttons */
  MenuSetActive (MenuAddHButton (GSTAT_BACK_LEFT,   MENU_BOTTOM, GSTAT_B_WIDTH, "<<<", ButtonBackward, par), 
		 (firstStat > 0) );
  MenuSetAbort  (MenuAddHButton (GSTAT_ESCAPE_LEFT, MENU_BOTTOM, GSTAT_B_WIDTH, "Up",  ButtonBack,     (void *) exitFunc) );
  MenuSetActive (MenuAddHButton (GSTAT_FORW_LEFT,   MENU_BOTTOM, GSTAT_B_WIDTH, ">>>", ButtonForward,  par),
		 (lastStat < numStat) );
  /* */
  MenuSetLinks ();
  /* that's all*/
  return XBFalse;
} /* CreateStatList */

/*
 * create single player statistics menu
 */
XBBool
CreateSinglePlayerStatMenu (void *par)
{
  XBStatData *ptr    = par;
  XBAtom      player = ptr->atom;

  /* delete old list */
  assert (datStat != NULL);
  free (datStat);
  /* get play list */
  datStat = CreatePlayerSingleStat (player, &numStat);
  if (NULL == datStat) {
    return CreatePlayerStatMenu (NULL);
  }
  firstStat  = 0;
  exitFunc   = CreatePlayerStatMenu;
  selectFunc = NULL;
  return CreateStatMenu ("Level");
} /* CreatePlayerStatMenu */

/*
 * create player statistics menu
 */
XBBool
CreatePlayerStatMenu (void *par)
{
  /* get play list */
  datStat = CreatePlayerTotalStat (&numStat);
  if (NULL == datStat) {
    return CreateExtrasMenu (NULL);
  }
  firstStat  = 0;
  exitFunc   = CreateExtrasMenu;
  selectFunc = CreateSinglePlayerStatMenu;
  return CreateStatMenu ("Player");
} /* CreatePlayerStatMenu */

/*
 * create single level statistics menu
 */
XBBool
CreateSingleLevelStatMenu (void *par)
{
  XBStatData *ptr    = par;
  XBAtom      level = ptr->atom;

  /* delete old list */
  assert (datStat != NULL);
  free (datStat);
  /* get play list */
  datStat = CreateLevelSingleStat (level, &numStat);
  if (NULL == datStat) {
    return CreateLevelStatMenu (NULL);
  }
  firstStat  = 0;
  exitFunc   = CreateLevelStatMenu;
  selectFunc = NULL;
  return CreateStatMenu ("Player");
} /* CreateLevelStatMenu */

/*
 * create level statistics menu
 */
XBBool
CreateLevelStatMenu (void *par)
{
  /* get play list */
  datStat = CreateLevelTotalStat (&numStat);
  if (NULL == datStat) {
    return CreateExtrasMenu (NULL);
  }
  firstStat  = 0;
  exitFunc   = CreateExtrasMenu;
  selectFunc = CreateSingleLevelStatMenu;
  return CreateStatMenu ("Level");
} /* CreateLevelStatMenu */

/*
 * demo button
 */ 
static XBBool
ButtonDemoStart (void *par)
{
  XBAtom *atom = par;

  assert (NULL != par);
  /* load demo */
  LoadDemoFromFile (*atom);
  /* start game in demo mode */
  SetHostType (XBPH_Demo);
  /* clean up */
  if (NULL != demoList) {
    free (demoList);
    demoList = NULL;
  }
  MenuClear ();
  /* let's go */
  return XBTrue;
} /* ButtonDemoStart */

/*
 * Forward in selection
 */
static XBBool
ButtonDemoForward (void *par)
{
  if (lastDemo < numDemos) {
    firstDemo = lastDemo;
  }
  return CreateDemoMenu (par);
} /* ButtonForward */

/*
 * Forward in selection
 */
static XBBool
ButtonDemoBackward (void *par)
{
  firstDemo -= MAX_GSTAT_ROWS;
  if (firstDemo < 0) {
    firstDemo = 0;
  }
  return CreateDemoMenu (par);
} /* ButtonForward */

/*
 * 
 */
static XBBool
CreateDemoMenu (void *par)
{
  size_t  i, j;

  /* create demo list if needed */
  if (NULL == demoList) {
    demoList = CreateDemoList (&numDemos);
    firstDemo = 0;
  }
  /* no demos >= back to extras */
  if (NULL == demoList) {
    return CreateExtrasMenu (NULL);
  }
  MenuClear ();
  /* Title */
  MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, "Demo Playback");
  /* list buttons */
  MenuSetActive (MenuAddDemoHeader (GSTAT_LEFT, GSTAT_TOP, GSTAT_WIDTH), XBFalse);
  for (i = 0, j = firstDemo; j < numDemos && i < MAX_GSTAT_ROWS; i ++, j ++) {
    (void) MenuAddDemoEntry (GSTAT_LEFT, GSTAT_ROW (i), GSTAT_WIDTH, demoList + j, ButtonDemoStart, &demoList[j].atom);
  }
  lastDemo = j;
  /* Buttons */
  MenuSetActive (MenuAddHButton (GSTAT_BACK_LEFT,   MENU_BOTTOM, GSTAT_B_WIDTH, "<<<",    ButtonDemoBackward, NULL), 
		 (firstDemo > 0) );
  MenuSetAbort  (MenuAddHButton (GSTAT_ESCAPE_LEFT, MENU_BOTTOM, GSTAT_B_WIDTH, "Extras", CreateExtrasMenu,   NULL) );
  MenuSetActive (MenuAddHButton (GSTAT_FORW_LEFT,   MENU_BOTTOM, GSTAT_B_WIDTH, ">>>",    ButtonDemoForward,  NULL), 
		 (lastDemo < numDemos) );
  /* */
  MenuSetLinks ();
  /* that's all*/
  return XBFalse;
} /* CreateDemoMenu */

/*
 * create the extras menu
 */
XBBool
CreateExtrasMenu (void *par)
{
  MenuClear ();
  /* Title */
  MenuAddLabel  (TITLE_LEFT, TITLE_TOP, TITLE_WIDTH, "Extras Menu");
  /* Buttons */				   
  (void) MenuAddHButton (MENU_LEFT, MENU_ROW (0), MENU_WIDTH, "Player Statistics", CreatePlayerStatMenu, NULL);
  (void) MenuAddHButton (MENU_LEFT, MENU_ROW (1), MENU_WIDTH, "Level Statistics",  CreateLevelStatMenu,  NULL);
  (void) MenuAddHButton (MENU_LEFT, MENU_ROW (2), MENU_WIDTH, "Demo Playback",     CreateDemoMenu,       NULL);
  /* leave menu */
  MenuSetAbort (MenuAddHButton (MENU_LEFT, MENU_BOTTOM, MENU_WIDTH, "Main Menu", CreateMainMenu, NULL) );
  /* escape */
  MenuSetLinks ();
  /* that's all*/
  return XBFalse;
} /* CreateExtrasMenu */

/*
 * end of file menu_extras.c
 */
