/*
 * file mi_host.c - Menu item for host selection in networked games
 *
 * $Id: mi_host.c,v 1.8 2001/07/01 11:17:53 xblast Exp $
 *
 * Program XBLAST 
 * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2; or (at your option)
 * any later version
 *
 * This program is distributed in the hope that it will be entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "mi_host.h"

#include "mi_map.h"

#include "gui.h"

/*
 * local macors
 */
#define FF_HOST_NAME_FOCUS     (FF_Small | FF_Black | FF_Center | FF_Outlined)
#define FF_HOST_NAME_NO_FOCUS  (FF_Small | FF_White | FF_Center)
#define FF_HOST_STATE_FOCUS    (FF_Large | FF_Black | FF_Center | FF_Outlined)
#define FF_HOST_STATE_NO_FOCUS (FF_Large | FF_White | FF_Center)

/*
 * local types
 */
typedef struct {
  XBMenuItem    item;
  const char  **pText;
  const char   *cText;
  XBHostState  *pState;
  XBHostState   cState;
  const int    *pPing;
  int           cPing;
  char          tPing[16];
  Sprite       *tSprite;
  Sprite       *sSprite;
  Sprite       *pSprite;
} XBMenuHostItem;

/*
 * local variables
 */
static XBHostState serverState = XBHS_Server;
static const char *stateText[NUM_XBHS] = {
  "?",    /* XBHS_None */
  "...",  /* XBHS_Wait */
  "In",   /* XBHS_In */
  "Out",  /* XBHS_Out */
  "Srv",  /* XBHS_Server */
};

/*
 * a host item receives the focus
 */
static void
MenuHostFocus (XBMenuItem *ptr, XBBool flag)
{
  XBMenuHostItem *host = (XBMenuHostItem *) ptr;

  assert (NULL != host);
  assert (NULL != host->tSprite);
  assert (NULL != host->sSprite);
  SetSpriteAnime (host->tSprite, flag ? FF_HOST_NAME_FOCUS  : FF_HOST_NAME_NO_FOCUS);
  SetSpriteAnime (host->sSprite, flag ? FF_HOST_STATE_FOCUS : FF_HOST_STATE_NO_FOCUS);
  if (NULL != host->pSprite) {
    SetSpriteAnime (host->pSprite, flag ? FF_HOST_NAME_FOCUS : FF_HOST_NAME_NO_FOCUS);
  }
} /* MenuToggleFocus */

/*
 * a server is selected
 */
static void
MenuServerSelect (XBMenuItem *ptr)
{
} /* MenuToggleSelect */

/*
 * a client is selected
 */
static void
MenuClientSelect (XBMenuItem *ptr)
{
  XBMenuHostItem *host = (XBMenuHostItem *) ptr;

  assert (NULL != host);
  assert (NULL != host->pState);
  switch (*host->pState) {
  case XBHS_In:  *host->pState = XBHS_Out; break;
  case XBHS_Out: *host->pState = XBHS_In;  break;
  default: break;
  }
} /* MenuToggleSelect */

/*
 * a peer is selected
 */
static void
MenuPeerSelect (XBMenuItem *ptr)
{
} /* MenuToggleSelect */

/*
 * mouse click
 */ 
static void
MenuServerMouse (XBMenuItem *ptr, XBEventCode code)
{
} /* MenuToggleMouse */

/*
 * mouse click
 */ 
static void
MenuClientMouse (XBMenuItem *ptr, XBEventCode code)
{
  if (code == XBE_MOUSE_1) {
    MenuClientSelect (ptr);
  }
} /* MenuToggleMouse */

/*
 * mouse click
 */ 
static void
MenuPeerMouse (XBMenuItem *ptr, XBEventCode code)
{
} /* MenuToggleMouse */


/*
 * polling a host item
 */
static void
MenuHostPoll (XBMenuItem *ptr)
{
  XBMenuHostItem *host = (XBMenuHostItem *) ptr;

  assert (NULL != host);
  assert (NULL != host->tSprite);
  assert (NULL != host->sSprite);
  /* state */
  if (*host->pState != host->cState) {
    host->cState = *host->pState;
    assert (host->cState < NUM_XBHS);
    SetSpriteText (host->sSprite, stateText[host->cState]);
  }
  /* name */
  if (*host->pText != host->cText) {
    host->cText = *host->pText;
    SetSpriteText (host->tSprite, host->cText);
  }
  /* ping */
  if (NULL != host->pPing &&
      *host->pPing != host->cPing) {
    host->cPing = *host->pPing;
    if (host->cPing < 0) {
      host->tPing[0] = 0;
    } else {
      sprintf (host->tPing, "%u ms", host->cPing);
    }
    SetSpriteText (host->pSprite, host->tPing);
  }
} /* MenuHostPoll */

/*
 * create server button
 */
XBMenuItem *
MenuCreateServer (int x, int y, int w, const char **pText)
{
  /* create item */
  XBMenuHostItem *host = calloc (1, sizeof (*host));
  assert (host != NULL);
  MenuSetItem (&host->item, MIT_Host, x, y, w, CELL_H, MenuHostFocus, MenuServerSelect, MenuServerMouse, MenuHostPoll);
  /* set server specific data */
  assert (pText != NULL);
  host->pText  =  pText;
  host->cText  = *pText;
  host->pState = &serverState;
  host->cState =  XBHS_None;
  /* sprite with host name */
  host->tSprite = CreateTextSprite (*pText, (x + 11) * BASE_X, y * BASE_Y, (w - 12) * BASE_X, CELL_H * BASE_Y,
				    FF_HOST_NAME_NO_FOCUS, SPM_MAPPED);
  /* sprite with host state */
  host->sSprite = CreateTextSprite (stateText[0], (x + 1) * BASE_X, y * BASE_Y, 9 * BASE_X, CELL_H * BASE_Y,
				    FF_HOST_STATE_NO_FOCUS, SPM_MAPPED);
  /* graphics */
  MenuAddLargeFrame (x / CELL_W, (x + w - 1) / CELL_W, y / CELL_H);
  return &host->item;
} /* MenuCreateServer */

/*
 * create server button
 */
XBMenuItem *
MenuCreateClient (int x, int y, int w, const char **pText, XBHostState *pState, const int *pPing)
{
  /* create item */
  XBMenuHostItem *host = calloc (1, sizeof (*host));
  assert (host != NULL);
  MenuSetItem (&host->item, MIT_Host, x, y, w, CELL_H, MenuHostFocus, MenuClientSelect, MenuClientMouse, MenuHostPoll);
  /* set server specific data */
  assert (pText  != NULL);
  assert (pState != NULL);
  host->pText  =  pText;
  host->cText  = *pText;
  host->pState =  pState;
  host->cState = *pState;
  host->pPing  =  pPing;
  host->cPing  = -1;
  /* sprite with host name */
  host->tSprite = CreateTextSprite (*pText, (x + 11) * BASE_X, (y+1) * BASE_Y, (w - 12) * BASE_X, (CELL_H/2) * BASE_Y,
				    FF_HOST_NAME_NO_FOCUS, SPM_MAPPED);
  /* sprite with ping to host */
  host->pSprite = CreateTextSprite (host->tPing, (x + 11) * BASE_X, (y+CELL_H/2) * BASE_Y, (w - 12) * BASE_X, (CELL_H/2-1) * BASE_Y,
				    FF_HOST_NAME_NO_FOCUS, SPM_MAPPED);
  /* sprite with host state */
  host->sSprite = CreateTextSprite (stateText[*pState], (x + 1) * BASE_X, y * BASE_Y, 9 * BASE_X, CELL_H * BASE_Y,
				    FF_HOST_STATE_NO_FOCUS, SPM_MAPPED);
  /* graphics */
  MenuAddLargeFrame (x / CELL_W, (x + w - 1) / CELL_W, y / CELL_H);
  return &host->item;
} /* MenuCreateServer */

/*
 * create server button
 */
XBMenuItem *
MenuCreatePeer (int x, int y, int w, const char **pText, XBHostState *pState, const int *pPing)
{
  /* create item */
  XBMenuHostItem *host = calloc (1, sizeof (*host));
  assert (host != NULL);
  MenuSetItem (&host->item, MIT_Host, x, y, w, CELL_H, MenuHostFocus, MenuPeerSelect, MenuPeerMouse, MenuHostPoll);
  /* set server specific data */
  assert (pText  != NULL);
  assert (pState != NULL);
  host->pText  =  pText;
  host->cText  = *pText;
  host->pState =  pState;
  host->cState = *pState;
  host->pPing  =  pPing;
  host->cPing  = -1;
  /* sprite with host name */
  host->tSprite = CreateTextSprite (*pText, (x + 11) * BASE_X, (y+1) * BASE_Y, (w - 12) * BASE_X, (CELL_H/2) * BASE_Y,
				    FF_HOST_NAME_NO_FOCUS, SPM_MAPPED);
  /* sprite with ping to host */
  host->pSprite = CreateTextSprite (host->tPing, (x + 11) * BASE_X, (y+CELL_H/2) * BASE_Y, (w - 12) * BASE_X, (CELL_H/2-1) * BASE_Y,
				    FF_HOST_NAME_NO_FOCUS, SPM_MAPPED);
  /* sprite with host state */
  host->sSprite = CreateTextSprite (stateText[*pState], (x + 1) * BASE_X, y * BASE_Y, 9 * BASE_X, CELL_H * BASE_Y,
				    FF_HOST_STATE_NO_FOCUS, SPM_MAPPED);
  /* graphics */
  MenuAddLargeFrame (x / CELL_W, (x + w - 1) / CELL_W, y / CELL_H);
  return &host->item;
} /* MenuCreateServer */

/*
 * delete a host 
 */
void
MenuDeleteHost (XBMenuItem *item)
{
  XBMenuHostItem *host = (XBMenuHostItem *) item;

  assert (NULL != host);
  assert (NULL != host->tSprite);
  assert (NULL != host->sSprite);
  DeleteSprite (host->tSprite);
  DeleteSprite (host->sSprite);
  if (NULL != host->pSprite) {
    DeleteSprite (host->pSprite);
  }
} /* DeleteButtonItem */

/*
 * end of file mi_host.c
 */
