/*
 * file player.c - ingame player mangment 
 *
 * $Id: player.c,v 1.43 2001/07/01 11:54:26 xblast Exp $
 *
 * Program XBLAST 
 * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2; or (at your option)
 * any later version
 *
 * This program is distributed in the hope that it will be entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "player.h"

#include "atom.h"
#include "geom.h"
#include "random.h"
#include "map.h"
#include "bomb.h"
#include "func.h"
#include "snd.h"
#include "str_util.h"
#include "status.h"
#include "info.h"

/*
 * local constants
 */
#define BOMB_STEP 2
#define STEP_HORI BASE_X
#define STEP_VERT BASE_Y

#define MAX_RANGE 10
#define ILLTIME 256
#define ILL_X (4*BASE_X)
#define ILL_Y (5*BASE_Y)

#define JUNKIE_ILL_TIME (ILLTIME)
#define JUNKIE_STUN_TIME 12
#define JUNKIE_TIME_1 360
#define JUNKIE_TIME_2 210
#define JUNKIE_TIME_3 60   /* Speed */

#define NUM_TELE_TRIES 25

/*
 * init flags for extras
 *   LF_ level start flag
 *   RF_ revive flag
 *   IF_ both of the above
 */
#define IF_None     0
/* kick extra */
#define LF_Kick     (1<<0)
#define RF_Kick     (1<<1)
#define IF_Kick     (RF_Kick|LF_Kick)
/* remote control */
#define LF_RC       (1<<2)
#define RF_RC       (1<<3)
#define IF_RC       (RF_RC|LF_RC)
/* teleporter */
#define LF_Teleport (1<<4)
#define RF_Teleport (1<<5)
#define IF_Teleport (RF_Teleport|LF_Teleport)
/* airpump */
#define LF_Airpump  (1<<6)
#define RF_Airpump  (1<<7)
#define IF_Airpump  (RF_Airpump|LF_Airpump)
/* cloak extra */
#define LF_Cloak    (1<<8)
#define RF_Cloak    (1<<9)
#define IF_Cloak    (RF_Cloak|LF_Cloak)
/* morph extra */
#define LF_Morph    (1<<10)
#define RF_Morph    (1<<11)
#define IF_Morph    (RF_Morph|LF_Morph)

#define NUM_IF 12


/*
 * global variables
 */
BMPlayer      player_stat[2*MAX_PLAYER];
PlayerStrings p_string[2*MAX_PLAYER];

/*
 * extern variables
 */
extern void (*special_extra_function)();

/*
 * local variables
 */
static int      minRange, minBombs;
static BMHealth reviveHealth;
static XBBool   randomPlayerPos;
static int      maxLives;
static int      numPlayer;
static int      specialBombs;
#if 0
static int game_mode;
#endif

/* conversion tables */
static DBToInt healthTable[] = {
  { "bomb",        (int) IllBomb },
  { "empty",       (int) IllEmpty },
  { "invisible",   (int) IllInvisible },
  { "malfunction", (int) IllMalfunction },
  { "mini",        (int) IllMini },
  { "reverse",     (int) IllReverse },
  { "run",         (int) IllRun },
  { "slow",        (int) IllSlow },
  { "teleport",    (int) IllTeleport },
  { NULL, Healthy },
};
static DBToInt initFlagsTable[] = {
 { "airpump",  LF_Airpump },
 { "cloak",    LF_Cloak },
 { "kick",     LF_Kick },
 { "morph",    LF_Morph },
 { "rc",       LF_RC },
 { "teleport", LF_Teleport },
 { NULL, 0 },
};
static DBToInt reviveFlagsTable[] = {
 { "airpump",  RF_Airpump },
 { "cloak",    RF_Cloak },
 { "kick",     RF_Kick },
 { "morph",    RF_Morph },
 { "rc",       RF_RC },
 { "teleport", RF_Teleport },
 { NULL, 0 },
};

/* info text data */
static const char *permHealthInfo[MAX_ILL] = {
  NULL,
  "Permanent random bombing",
  "Permanent slowdown",
  "Permanent running",
  "Permanent mini bombs",
  "No bombs while healthy",
  "Permanent invisibility",
  "Permanent malfunctions",
  "Permanent reverse controls",
  "Permanent random teleporting",
};
static const char *initHealthInfo[MAX_ILL] = {
  NULL,
  "Initial random bombing",
  "Initial slowdown",
  "Initial running",
  "Initial mini bombs",
  "No bombs while healthy",
  "Initial invisibility",
  "Initial malfunctions",
  "Initial reverse controls",
  "Initial random teleporting",
};
static const char *reviveHealthInfo[MAX_ILL] = {
  NULL,
  "Revived with random bombing",
  "Revived with slowdown",
  "Revived with running",
  "Revived with mini bombs",
  "Revived with bombs while healthy",
  "Revived with invisibility",
  "Revived with malfunctions",
  "Revived with reverse controls",
  "Revived with random teleporting",
};


/*
 * evaluate level data 
 */
XBBool
ConfigLevelPlayers (const DBSection *section, XBBool allowRandomPos)
{
  BMPlayer   *ps;
  int         i, j;
  BMPosition  pos[MAX_PLAYER];
  const char *s;
  BMHealth    initHealth;
  unsigned    initFlags, flags;

  assert (section != NULL);

  if (! DB_GetEntryInt (section, atomBombs, &minBombs) ) {
    return XBFalse;
  }
  if (! DB_GetEntryInt (section, atomRange, &minRange) ) {
    return XBFalse;
  }
  if (! DB_GetEntryInt (section, atomSpecialBombs, &specialBombs) ) {
    specialBombs = 0;
  }
  if (! DB_ConvertEntryInt (section, atomReviveVirus, (int *) &reviveHealth, healthTable) ) {
    reviveHealth = Healthy;
  }
  if (! DB_ConvertEntryInt (section, atomInitVirus, (int *) &initHealth, healthTable) ) {
    initHealth = Healthy;
  }
  /* init flags */
  initFlags = 0;
  if (DB_ConvertEntryFlags (section, atomInitExtra, &flags, initFlagsTable) ) {
    initFlags |= flags; 
  }
  if (DB_ConvertEntryFlags (section, atomReviveExtra, &flags, reviveFlagsTable) ) {
    initFlags |= flags; 
  }
  /* retrieve all player positions */
  for (i = 0; i < MAX_PLAYER; i ++) {
    /* get position */
    if (! DB_GetEntryPos (section, atomArrayPos0[i+1], pos + i) ) {
      return XBFalse;
    }
  }  
  /* shuffle player positions if needed */
  if (allowRandomPos && randomPlayerPos) {
    /* determine number of positions to shuffle */
    int numShuffle = 0;
    for (i = 0; i < numPlayer; i ++) {
      if (! player_stat[i].in_active) {
	numShuffle ++;
      }   
    } 
    numShuffle = (numShuffle <= 4) ? 4 : 6;
    /* shuffle them */
    for (i = 0; i < numShuffle; i ++) {
      BMPosition swap;
      j = GameRandomNumber (numShuffle-1);
      if (j >= i) {
	j ++;
      }
      Dbg_Out ("shuffle %d<->%d\n", i, j);
      swap   = pos[i];
      pos[i] = pos[j];
      pos[j] = swap;
    }
  }
  /* set player positions */
  j = 0;
  for (i = 0; i < MAX_PLAYER; i ++) {
    ps = player_stat + i;
    if (ps->in_active) {
      ps->x = 0;
      ps->y = i * BLOCK_HEIGHT;
      SetSpriteMode (ps->sprite, SPM_UNMAPPED);
    } else {
      ps->x =  pos[j].x    * BLOCK_WIDTH;
      ps->y = (pos[j].y-1) * BLOCK_HEIGHT;
      j ++;
    }
  }
  /* setup other player attribtes */
  for (i = 0; i < numPlayer; i ++) {
    ps = player_stat + i;
    /* --- */
    ps->invincible     = NEW_INVINCIBLE;
    ps->illness        = initHealth;
    ps->health         = initHealth;
    ps->illtime        = 0;
    ps->junkie         = 0;
    ps->dying          = 0;
    ps->stunned        = 0;
    ps->lives          = ps->in_active ? 0 : maxLives;
    ps->range          = minRange;
    ps->bombs          = minBombs;
    ps->extra_flags    = initFlags;
    ps->special_bombs  = specialBombs;
    ps->remote_control = (LF_RC       & ps->extra_flags) ? XBTrue : XBFalse;
    ps->teleport       = (LF_Teleport & ps->extra_flags) ? XBTrue : XBFalse;
    ps->kick           = (LF_Kick     & ps->extra_flags) ? XBTrue : XBFalse;
    ps->air_button     = (LF_Airpump  & ps->extra_flags) ? XBTrue : XBFalse;
    ps->cloaking       = (LF_Cloak    & ps->extra_flags) ? -GAME_TIME : 0;
    ps->num_extras     = 0;  
    ps->abort 	       = ABORT_NONE;
    ps->d_ist 	       = GoStop;
    ps->d_soll	       = GoStop;
    ps->d_look	       = GoDown;
    ps->num_morph      = 0;
    ps->morphed        = XBFalse;
  }
  /* set text for info screen */
  switch (minBombs) {
  case 0:  AddPlayerInfo ("No bombs");           break;
  case 1:  AddPlayerInfo ("1 bomb");             break;
  default: AddPlayerInfo ("%d bomb", minBombs); break;
  }
  switch (minRange) {
  case 0:  AddPlayerInfo ("No initial range");           break;
  case 1:  AddPlayerInfo ("No mini bombs");              break;
  default: AddPlayerInfo ("Initial range %d", minRange); break;
  }
  if (initHealth == reviveHealth) {
    if (NULL != (s = permHealthInfo[initHealth]) ) {
      AddPlayerInfo (s);
    }
  } else {
    if (NULL != (s = initHealthInfo[initHealth]) ) {
      AddPlayerInfo (s);
    }
    if (NULL != (s = reviveHealthInfo[reviveHealth]) ) {
      AddPlayerInfo (s);
    }
  }
  switch (initFlags & IF_Airpump) {
  case IF_Airpump: AddPlayerInfo ("Airpump as default");   break;
  case LF_Airpump: AddPlayerInfo ("Initital airpump");     break;
  case RF_Airpump: AddPlayerInfo ("Revived with airpump"); break;
  default:         break;
  }
  switch (initFlags & IF_Cloak) {
  case IF_Cloak: AddPlayerInfo ("Cloak as default");   break;
  case LF_Cloak: AddPlayerInfo ("Initital cloak");     break;
  case RF_Cloak: AddPlayerInfo ("Revived with cloak"); break;
  default:         break;
  }
  switch (initFlags & IF_Kick) {
  case IF_Kick: AddPlayerInfo ("Kick as default");   break;
  case LF_Kick: AddPlayerInfo ("Initital kick");     break;
  case RF_Kick: AddPlayerInfo ("Revived with kick"); break;
  default:         break;
  }
  switch (initFlags & IF_Morph) {
  case IF_Morph: AddPlayerInfo ("Morphing as default");   break;
  case LF_Morph: AddPlayerInfo ("Initital morphing");     break;
  case RF_Morph: AddPlayerInfo ("Revived with morphing"); break;
  default:         break;
  }
  switch (initFlags & IF_RC) {
  case IF_RC: AddPlayerInfo ("Remote control as default");   break;
  case LF_RC: AddPlayerInfo ("Initital remote control");     break;
  case RF_RC: AddPlayerInfo ("Revived with remote control"); break;
  default:         break;
  }
  switch (initFlags & IF_Teleport) {
  case IF_Teleport: AddPlayerInfo ("Teleport as default");   break;
  case LF_Teleport: AddPlayerInfo ("Initital teleport");     break;
  case RF_Teleport: AddPlayerInfo ("Revived with teleport"); break;
  default:         break;
  }
  /* that's all folks */
  return XBTrue;
} /* ConfigLevelPlayers */

/*
 * Create Welcome messages for players at start of level
 */
void
WelcomePlayers (void)
{
  int i, j, num;
  const char *list[MAX_PLAYER];
  const char *swap;

  /* get messages */
  for (i = 0, num = 0; i < numPlayer; i ++) {
    if (NULL != p_string[i].welcome &&
	! player_stat[i].in_active) {
      list[num ++] = p_string[i].welcome;
    }
  }
  /* shuffle them */
  for (i = 0; i < num; i ++) {
    j       = OtherRandomNumber (num);
    swap    = list[i];
    list[i] = list[j];
    list[j] = swap;
  }
  /* show them */
  for (i = 0; i < num; i ++) {
    SetMessage (list[i], XBFalse);
  }
} /* WelcomePlayers */

/*
 *
 */
int
NumSpecialBombs (void)
{
  return specialBombs;
} /* NumSpecialBombs */

/*
 * set player stat to default values
 */
static void
InitPlayerStat (BMPlayer *ps, int player, int ctrl, XBPlayerTeam team, XBBool local)
{
  /* set default values */
  ps->victories = 0;
  ps->id        = player;
  ps->number    = player;
  ps->disp      = local ? SPM_MAPPED : SPM_UNMAPPED;
  ps->sprite    = CreatePlayerSprite (ps->number, 0, 0, 0, SPM_UNMAPPED);
  ps->in_active = XBFalse;
  /* evaluate team mode */
  if (team == XBPT_None) {
    ps->team = ps->id;
  } else {
    ps->team = team - XBPT_None -1;
  }
} /* InitPlayerStat */

/*
 * set all messages for given player
 */
static void
InitPlayerMessages (PlayerStrings *str, const CFGPlayer *cfgPlayer)
{
  char tmp[128];

  /* player name */
  str->name  = DupString (cfgPlayer->name);
  assert (NULL != str->name);
  str->tag   = DupString (cfgPlayer->name);
  assert (NULL != str->tag);
  /* pause string */
  sprintf (tmp, "Game paused by %s", str->name);
  str->pause = DupString (tmp);
  assert (NULL != str->pause);
  /* win a level */
  if (NULL != cfgPlayer->messages.msgWinLevel) {
    str->winlevel = DupString (cfgPlayer->messages.msgWinLevel);
  } else {
    sprintf (tmp, "%s wins", str->name);
    str->winlevel = DupString (tmp);
  }
  assert (NULL != str->winlevel);
  /* win the game */
  if (NULL != cfgPlayer->messages.msgWinGame) {
    str->wingame = DupString (cfgPlayer->messages.msgWinGame);
  } else {
    str->wingame = DupString ("CONGRATULATIONS!");
  }
  assert (NULL != str->wingame);
  /* request abort */
  sprintf (tmp, "Abort requested by %s", str->tag);
  str->abort = DupString (tmp); 
  assert (NULL != str->abort);
  /* cancel abort */
  sprintf (tmp, "%s cancels abort", str->tag);
  str->abortcancel = DupString (tmp); 
  assert (str->abortcancel != NULL);
  /* loosing a life */
  if (NULL != cfgPlayer->messages.msgLoseLife) {
    str->loselife = DupString (cfgPlayer->messages.msgLoseLife);
  } else {
    str->loselife = NULL;
  }
  /* loosing a level */
  if (NULL != cfgPlayer->messages.msgLoseLevel) {
    str->loselevel = DupString (cfgPlayer->messages.msgLoseLevel);
  } else {
    str->loselevel = NULL;
  }
  /* gloating */
  if (NULL != cfgPlayer->messages.msgGloat) {
    str->gloat = DupString (cfgPlayer->messages.msgGloat);
  } else {
    str->gloat = NULL;
  }
  /* welcome to the game */
  if (NULL != cfgPlayer->messages.msgWelcome) {
    str->welcome = DupString (cfgPlayer->messages.msgWelcome);
  } else {
    str->welcome = NULL;
  }
} /* InitPlayerMessages */

/*
 *
 */
void
InitPlayers (XBPlayerHost host, const CFGGame *cfgGame, const CFGPlayer *cfgPlayer)
{
  int  i;

  assert (NULL != cfgGame);
  assert (NULL != cfgPlayer);
  
  /* global settings */
  numPlayer       = cfgGame->players.num;
  maxLives        = cfgGame->setup.numLives;
  randomPlayerPos = cfgGame->setup.randomPlayers;
#if 0
  game_mode  = g_mode;
#endif

  /* player settings */
  for (i = 0; i < cfgGame->players.num; i ++) {
    assert (ATOM_INVALID != cfgGame->players.player[i]);
    InitPlayerStat (player_stat + i, i, i, cfgGame->players.team[i], (host == cfgGame->players.host[i]) );
    InitPlayerMessages (p_string + i, cfgPlayer + i);
  }
} /* InitPlayers */

/*
 *
 */
void
FinishPlayers (void)
{
  int i;
  PlayerStrings *str;

  for (i = 0; i < numPlayer; i ++) {
    str = p_string + i; 
    if (NULL != str->name) {
      free (str->name);
    }
    if (NULL != str->tag) {
      free (str->tag);
    }
    if (NULL != str->pause) {
      free (str->pause);
    }
    if (NULL != str->winlevel) {
      free (str->winlevel);
    }
    if (NULL != str->wingame) {
      free (str->wingame);
    }
    if (NULL != str->loselife) {
      free (str->loselife);
    }
    if (NULL != str->loselevel) {
      free (str->loselevel);
    }
    if (NULL != str->gloat) {
      free (str->gloat);
    }
    if (NULL != str->welcome) {
      free (str->welcome);
    }
    if (NULL != str->abort) {
      free (str->abort);
    }
    if (NULL != str->abortcancel) {
      free (str->abortcancel);
    }
  }
  numPlayer = 0;
} /* FinishPlayers */

/*
 *
 */
void
DeletePlayerSprites (void)
{
  int player;

  for (player=0; player<numPlayer; player++) {
    DeleteSprite (player_stat[player].sprite);
  }
} /* DeletePlayerSprites */

/* 
 *
 */
void 
DropBomb (BMPlayer *ps, int type)
{
  if ( (ps->bombs !=0) && 
       (ps->illness!=IllEmpty) && 
       (ps->morphed < 2) &&
       (type == BMTdefault || ps->special_bombs > 0) ) {
    if (ps->lives >0) {
      if (NewPlayerBomb (ps, type)) {
	SND_Play (SND_DROP, ps->x / (PIXW / MAX_SOUND_POSITION));
	ps->bombs --;
	if (ps->morphed) {
	  ps->morphed = 2;
	  ps->num_morph --;
	}
	if (type != BMTdefault) {
	  ps->special_bombs --;
	}
      } else {
	if (ps->morphed) {
	  ps->morphed = 0;
	}
      }
    }
  }
} /* DropBomb */

/* 
 *
 */
static void 
WalkStop (BMPlayer *ps, int flag, int mazex, int mazey)
{
  if (ps->illness != IllReverse) {
    switch(ps->d_look) {
    case GoDown:
      SetSpriteAnime (ps->sprite,SpriteStopDown);
      break;
    case GoUp:
      SetSpriteAnime (ps->sprite,SpriteStopUp);
      break;
    case GoLeft:
      SetSpriteAnime (ps->sprite,SpriteStopLeft);
      break;
    case GoRight:
      SetSpriteAnime (ps->sprite,SpriteStopRight);
      break;
    default:
      break;
    }
  } else {
    switch(ps->d_look) {
    case GoDown:
      SetSpriteAnime(ps->sprite,SpriteStopUp);
      break;
    case GoUp:
      SetSpriteAnime(ps->sprite,SpriteStopDown);
      break;
    case GoLeft:
      SetSpriteAnime(ps->sprite,SpriteStopRight);
      break;
    case GoRight:
      SetSpriteAnime(ps->sprite,SpriteStopLeft);
      break;
    default:
      break;
    }
  }
} /* WalkStop */



/* 
 *
 */
static void 
WalkUp (BMPlayer *ps, int flag, int mazex, int mazey)
{
  if ( !(flag && CheckMaze(mazex, mazey-1))) {
    ps->y -= STEP_VERT;
    mazey = ps->y/BLOCK_HEIGHT+1;
    if (ps->illness != IllReverse) {
      SetSpriteAnime(ps->sprite, SpriteWalkUp0 + ((ps->y/STEP_VERT) % 4));
    } else {
      SetSpriteAnime(ps->sprite, SpriteWalkDown0 + ((ps->y/STEP_VERT) % 4));
    }
  } else {
    ps->d_ist = GoStop;
    if (ps->illness != IllReverse) {
      SetSpriteAnime(ps->sprite, SpriteStopUp);
    } else {
      SetSpriteAnime(ps->sprite, SpriteStopDown);
    }
  }

  /* try a kick */
  if (CheckBomb (mazex, mazey) 
      && ( (ps->y % BLOCK_HEIGHT) == (STEP_VERT*BOMB_STEP) ) ) {
    if (ps->kick) {
      SND_Play (SND_KICK, ps->x / (PIXW / MAX_SOUND_POSITION));
      MoveBomb (mazex, mazey, GoUp);
      ps->d_soll = GoStop;
    }
    ps->y += STEP_VERT;
  }
} /* WalkUp */

/* 
 * local function walk_left 
 */
static void 
WalkLeft (BMPlayer *ps, int flag, int mazex, int mazey)
{
  if ( !(flag && CheckMaze(mazex -1, mazey) )) {
    ps->x -= STEP_HORI;
    mazex = ps->x/BLOCK_WIDTH;
    if (ps->illness != IllReverse) {
      SetSpriteAnime(ps->sprite, SpriteWalkLeft0 + ((ps->x/STEP_HORI) % 4));
    } else {
      SetSpriteAnime(ps->sprite, SpriteWalkRight0 + ((ps->x/STEP_HORI) % 4));
    }
  } else {
    ps->d_ist = GoStop;
    if (ps->illness != IllReverse) {
      SetSpriteAnime(ps->sprite, SpriteStopLeft);
    } else {
      SetSpriteAnime(ps->sprite, SpriteStopRight);
    }
  }

  /* try a kick */
  if ( CheckBomb(mazex, mazey) 
      && ((ps->x % BLOCK_WIDTH) == (STEP_HORI*BOMB_STEP) ) ) {
    if (ps->kick) {
     SND_Play (SND_KICK, ps->x / (PIXW / MAX_SOUND_POSITION));
     MoveBomb (mazex, mazey, GoLeft);
      ps->d_soll = GoStop;
    }
    ps->x += STEP_HORI;
  }
} /* WalkLeft */

/* 
 *
 */
static void 
WalkDown (BMPlayer *ps, int flag, int mazex, int mazey)
{
  if ( !(flag && CheckMaze(mazex, mazey+1) )) {
    ps->y += STEP_VERT;
    mazey = ps->y/BLOCK_HEIGHT + 1;
    if (ps->illness != IllReverse) {
      SetSpriteAnime(ps->sprite, SpriteWalkDown0 + ((ps->y/STEP_VERT) % 4));
    } else {
      SetSpriteAnime(ps->sprite, SpriteWalkUp0 + ((ps->y/STEP_VERT) % 4));
    }
  } else {
    ps->d_ist = GoStop;
    if (ps->illness != IllReverse) {
      SetSpriteAnime(ps->sprite, SpriteStopDown);
    } else {
      SetSpriteAnime(ps->sprite, SpriteStopUp);
    }
  }

  /* try a kick */
  if ( CheckBomb(mazex, mazey+1)
      && ( (ps->y % BLOCK_HEIGHT) == (BLOCK_HEIGHT - STEP_VERT*BOMB_STEP) )
      ) {
    if (ps->kick) {
      SND_Play (SND_KICK, ps->x / (PIXW / MAX_SOUND_POSITION));
      MoveBomb(mazex, mazey+1,	GoDown);
      ps->d_soll = GoStop;
    }
    ps->y -= STEP_VERT;
  }
} /* WalkDown */

/* 
 *
 */
static void 
WalkRight (BMPlayer *ps, int flag, int mazex, int mazey)
{
  if ( !(flag && CheckMaze(mazex +1, mazey))) {
    ps->x += STEP_HORI;
    mazex = ps->x/BLOCK_WIDTH;
    if (ps->illness != IllReverse) {
      SetSpriteAnime(ps->sprite, SpriteWalkRight0 + ((ps->x/STEP_HORI) % 4));
    } else {
      SetSpriteAnime(ps->sprite, SpriteWalkLeft0 + ((ps->x/STEP_HORI) % 4));
    }
  } else {
    ps->d_ist = GoStop;
    if (ps->illness != IllReverse) {
      SetSpriteAnime(ps->sprite, SpriteStopRight);
    } else {
      SetSpriteAnime(ps->sprite, SpriteStopLeft);
    }
  }

  /* try kick */
  if ( CheckBomb(mazex+1, mazey)
      && ( (ps->x % BLOCK_WIDTH) == (BLOCK_WIDTH - STEP_HORI*BOMB_STEP) ) ) {
    if (ps->kick) {
      SND_Play (SND_KICK, ps->x / (PIXW / MAX_SOUND_POSITION));
      MoveBomb(mazex+1, mazey, GoRight);
      ps->d_soll = GoStop;
    }
    ps->x -= STEP_HORI;
  }
} /* WalkRight */

/*
 * try to teleport player
 */
static XBBool
TeleportPlayer (BMPlayer *ps, int mazeX, int mazeY) 
{
  int newMazeX, newMazeY;
  int i;
  
  for (i = 0; i < NUM_TELE_TRIES; i ++) {
    newMazeX = GameRandomNumber (MAZE_W);
    newMazeY = GameRandomNumber (MAZE_H);
    if (! CheckMaze (newMazeX, newMazeY) && 
	(mazeX != newMazeX || mazeY != newMazeY) ) {
      SND_Play (SND_TELE1, ps->x / (PIXW / MAX_SOUND_POSITION));
      ps->x      =  newMazeX    * BLOCK_WIDTH;
      ps->y      = (newMazeY-1) * BLOCK_HEIGHT;
      ps->d_soll = GoStop;
      ps->d_look = GoDown;
      SND_Play (SND_TELE2, ps->x / (PIXW / MAX_SOUND_POSITION));
      return XBTrue;
    }
  }
  return XBFalse;
} /* TeleportPlayer */


/* 
 * local function do_walk 
 */
static void 
DoWalk (BMPlayer *ps, int gameTime)
{
  XBBool flag;
  int mazeX, mazeY;
  int i;
  int xalt,yalt;
  int spm_mode;

  xalt = ps->x;
  yalt = ps->y;

  if (ps->illness != IllSlow || 0 == gameTime % 2) {
    for (i = 0; i <= (ps->illness == IllRun); i ++) {
      flag  = XBFalse;
      mazeX = ps->x / BLOCK_WIDTH;
      mazeY = ps->y / BLOCK_HEIGHT + 1;
      
      if ( 0 == (ps->x % BLOCK_WIDTH) && 
	   0 == (ps->y % BLOCK_HEIGHT) ) {
	flag = XBTrue;
	/* check if player has deliberately teleported */
	if (ps->teleport == TELEPORT_TIME) {
	  if (TeleportPlayer (ps, mazeX, mazeY)) {
	    ps->teleport --;
	  }
	}
	/* change direction if needed */
	ps->d_ist = ps->d_soll;
	if (ps->d_ist != GoStop) {
	  ps->d_look = ps->d_ist;
	}
      }
      /* random teleporting */
      if ( (ps->illness == IllTeleport) && 
	   (0 == GameRandomNumber (32) ) ) {
	TeleportPlayer (ps, mazeX, mazeY);
	ps->d_ist  = GoStop;
	ps->d_soll = GoStop;
      }
      /* let the player walk */
      switch (ps->d_ist) {
      case GoStop:  WalkStop  (ps, flag, mazeX, mazeY); break;
      case GoLeft:  WalkLeft  (ps, flag, mazeX, mazeY); break;
      case GoRight: WalkRight (ps, flag, mazeX, mazeY); break;
      case GoDown:  WalkDown  (ps, flag, mazeX, mazeY); break;
      case GoUp:    WalkUp    (ps, flag, mazeX, mazeY); break;
      default:      break;
      }
      MoveSprite (ps->sprite, ps->x, ps->y);

      /* insert get _extra here */
      if ( (ps->x % BLOCK_WIDTH == 0) && (ps->y % BLOCK_HEIGHT == 0) ) {
	switch(GetExtra(ps->invincible, ps->x / BLOCK_WIDTH , ps->y / BLOCK_HEIGHT + 1 )) {
	case BTBomb:
	  SND_Play (SND_NEWBOMB, ps->x / (PIXW / MAX_SOUND_POSITION));
	  ps->bombs ++;
	  break;
	case BTRange:
	  SND_Play (SND_MOREFIRE, ps->x / (PIXW / MAX_SOUND_POSITION));
	  if (ps->range < MAX_RANGE) {
	    ps->range ++;
	  }
	  break;
	case BTSick:
	  ps->illtime = ILLTIME;
	  ps->illness = GameRandomNumber (MAX_ILL) + 1;
	  if (ps->illness == IllInvisible) {
	    SND_Play (SND_INVIS, ps->x / (PIXW / MAX_SOUND_POSITION));
	  } else {
	    SND_Play (SND_BAD, ps->x / (PIXW / MAX_SOUND_POSITION));
	  } 
	  if (ps->illness == IllReverse) {
	    switch (ps->d_ist) {
	    case GoDown:  ps->d_ist = GoUp;    break;
	    case GoUp:    ps->d_ist = GoDown;  break;
	    case GoLeft:  ps->d_ist = GoRight; break;
	    case GoRight: ps->d_ist = GoLeft;  break;
	    default:      break;
	    }
	  }
	  break;
	  
	case BTSpecial:
	  ps->num_extras ++;
	  (*specialExtraFunc) (ps);
	  break;
	}
      }
    }
  }

  /* draw player if not totally invisible or morphed */
  if (ps->in_active) {
    spm_mode = SPM_UNMAPPED;
  } else if (ps->illness != IllInvisible) {
    /* set default mode */
    spm_mode = SPM_MAPPED;
    /* first check for cloak */
    if (ps->cloaking < 0) {
      ps->cloaking ++;
      if (ps->cloaking & 0x01) {
	spm_mode = ps->disp;
      } else {
	spm_mode = SPM_UNMAPPED;
      }
    }
    /* blinking if invincible */
    if(ps->invincible > 0) {
      ps->invincible --;
      if (ps->invincible & 0x01) {
	spm_mode |= SPM_MASKED;
      }
      /* or slower blinking if arrived from teleport */
    } else if (ps->teleport > 1) {
      ps->teleport --;
      if ( (ps->teleport>>1) & 0x01) {
	spm_mode |= SPM_MASKED;
      }
    }
  } else {
    spm_mode = SPM_UNMAPPED;
  }
  SetSpriteMode (ps->sprite, spm_mode);

  /* is player still sick? */
  if (ps->illness != ps->health) {
    /* decrement illness timer */
    if ( (ps->illtime --) == 0) {
      /* heal if time is over */
      ps->illness = ps->health;
    }
  }

  /* drop random bombs if needed */
  if ( (ps->x % BLOCK_WIDTH == 0) && (ps->y % BLOCK_HEIGHT == 0) ) {
    if (ps->illness == IllBomb) {
      if (GameRandomNumber(4) != 0) {
	DropBomb (ps, BMTdefault);
      }
    }
  }
} /* DoWalk */

/* 
 * 
 */
static void 
DoMorph (BMPlayer *ps)
{
  SetSpriteAnime (ps->sprite, SpriteMorphed);
  SetSpriteMode (ps->sprite, (ps->morphed == 2) ? SPM_MAPPED : SPM_UNMAPPED);
  if (ps->d_soll != GoStop) {
    MoveBomb (ps->x / BLOCK_WIDTH, ps->y / BLOCK_HEIGHT + 1, ps->d_soll);
    ps->d_soll = GoStop;
  }
} /* DoMorph */

/*
 * 
 */
void
DoJunkie (void)
{
  BMPlayer *ps1;

  /* Junkie countdown */
  for (ps1 = player_stat; ps1 < player_stat + numPlayer; ps1++) {
    if ((ps1->lives) && (ps1->junkie)) {
      /* Junkie sickness */
      switch (--(ps1->junkie)) {
      case JUNKIE_TIME_1:
      case JUNKIE_TIME_2:
        /* Give a random illness */
        ps1->illtime = JUNKIE_ILL_TIME;
	ps1->illness = GameRandomNumber (MAX_ILL) + 1;
	break;
	
      case JUNKIE_TIME_3:
        /* Stun player and give speed */
        ps1->stunned += JUNKIE_STUN_TIME;
        ps1->illtime  = JUNKIE_ILL_TIME;
        ps1->illness  = IllRun;
	break;

      case 0:
        /* Too long! Take a hit. */
        ps1->dying  = DEAD_TIME;
        ps1->junkie = MAX_JUNKIE_TIME;
	break;
      }
    }
  }
} /* DoJunkie */

/*
 *
 */
void 
InfectOtherPlayers (void)
{
  BMPlayer *ps1,*ps2;

  for (ps1 = player_stat; ps1 < player_stat+numPlayer; ps1 ++) {
    for (ps2= ps1+1; ps2 < player_stat+numPlayer; ps2 ++) {
      if ( (ABS(ps1->x - ps2->x) < ILL_X)
	  && (ABS(ps1->y - ps2->y) < ILL_Y)) {
	/* infection with "normal" viruses */
	if (ps1->illness != ps2->illness) {
	  if ( (! ps2->invincible) &&  (ps1->illtime > ps2->illtime) ) {
	    ps2->illness = ps1->illness;
	    ps2->illtime = ILLTIME;
	  } else if ( (! ps1->invincible) &&  (ps2->illtime > ps1->illtime) ) {
	    ps1->illness = ps2->illness;
	    ps1->illtime = ILLTIME;
	  }
	}
	/* infection with junkie virus */
	if ( ( (ps2->junkie) && (!ps1->invincible) ) || (ps1->junkie) ) {
	  ps1->junkie = MAX_JUNKIE_TIME;
	}
	if ( ( (ps1->junkie) && (!ps2->invincible) ) || (ps2->junkie) ) {
	  ps2->junkie = MAX_JUNKIE_TIME;
	} 
      }
    }
  }
} /* InfectOtherPlayers */

/*
 *
 */
static void 
HaveAGloat (int player)
{
  int g, gloatpl, gloatpltt;
  
  gloatpl = -1;
  for (g = 0; g < 6; g ++) {
    gloatpltt = OtherRandomNumber(numPlayer);
    if (gloatpltt != player && 
	player_stat[gloatpltt].lives > 0) {
      gloatpl = gloatpltt;
      break;
    }
  }      
  if(gloatpl > -1) {
    SetMessage (p_string[gloatpl].gloat, XBFalse);
  }
} /* HaveAGloat */

/*
 * 
 */
void
KillPlayerAt (int x, int y)
{
  BMPlayer *ps;
  int player;

  for (player = 0; player < numPlayer; player ++) {
    ps = player_stat + player;
    if ( ps->lives > 0 ) {
      if (    (ps->x < (x+1)*BLOCK_WIDTH)
           && (ps->x > (x-1)*BLOCK_WIDTH)
           && (ps->y < (y)  *BLOCK_HEIGHT)
           && (ps->y > (y-2)*BLOCK_HEIGHT) ) {
        ps->lives = 1;
        ps->dying = DEAD_TIME;
      }
    }
  }
} /* KillPlayerAt */

/*
 *
 */
int
KillOtherPlayers (int team)
{
  int count = 0;
  int player;

  for (player = 0; player <numPlayer; player ++) {
    if ( (player_stat[player].team != team) 
	 && (player_stat[player].lives > 0) ) {
      player_stat[player].dying= DEAD_TIME;
      count ++;
    }
  }

  return count;
} /* KillOtherPlayers */

/*
 *
 */
int
StunOtherPlayers (int team, int time)
{
  int count = 0;
  int player;

  for (player = 0; player <numPlayer; player ++) {
    if ( (player_stat[player].team != team) 
	 && (! player_stat[player].invincible > 0) ) {
      SND_Play (SND_STUN, player_stat[player].x / (PIXW / MAX_SOUND_POSITION));
      player_stat[player].stunned = time;
      count ++;
    }
  }
  return count;
} /* StunOtherPlayers */

/*
 * 
 */
static void 
RevivePlayer (BMPlayer *ps, int *active_player)
{
  BMPlayer *ptr;
  PlayerStrings *st;
  int i, team_alive;
  
  st = p_string + ps->id;

  ps->lives --;
  /* check if player has lost all lives? */
  if (ps->lives == 0) {
    SetSpriteMode (ps->sprite, SPM_UNMAPPED);
    SND_Play (SND_DEAD, ps->x / (PIXW / MAX_SOUND_POSITION));
    team_alive = XBFalse;
    for (i = 0, ptr = player_stat; i<numPlayer; i++, ptr++) {
      if (ptr->team == ps->team) {
	team_alive |= (ptr->lives != 0);
      }
    }
    if (!team_alive) {
      (*active_player) --;
    }
    DistributeExtras (ps->bombs - minBombs, ps->range - minRange, ps->num_extras, ps->special_bombs);
    SetMessage (st->loselevel, XBFalse);
  }  else {
    SND_Play (SND_OUCH, ps->x / (PIXW / MAX_SOUND_POSITION));
    DistributeExtras (0, 0, ps->num_extras, ps->special_bombs);
    SetMessage (st->loselife, XBFalse);
  }
  HaveAGloat (ps->id);
  /* reset values */
  ps->invincible = NEW_INVINCIBLE;
  ps->dying      = 0;
  ps->stunned    = 0;
  ps->illness    = reviveHealth;
  ps->health     = reviveHealth;
  ps->illtime    = 0;
  ps->teleport   = 0;
  ps->cloaking   = XBFalse;
  ps->morphed    = XBFalse;
  ps->num_morph  = 0;
  /* Note that junkie ISN'T reset (not a bug) */
  /* very important */
  if (ps->remote_control > 0) {
    IgnitePlayersBombs (ps);
  }
  ps->remote_control = XBFalse;
  ps->kick           = XBFalse;
  ps->air_button     = XBFalse;
  /* If special bombs are distributed, then zero the count */
  if (DistribSpecial()) {
    ps->special_bombs = 0;
  }
  /* Reset extra pickup count */
  ps->num_extras = 0;
  /* reset inital extras */
  if (RF_RC & ps->extra_flags) {
    ps->remote_control = 1;
  }
  if (RF_Teleport & ps->extra_flags) {
    ps->teleport = 1;
  }
  if (RF_Kick & ps->extra_flags) {
    ps->kick = 1;
  }
  if (RF_Airpump & ps->extra_flags) {
    ps->air_button = 1;
  }
  if (RF_Cloak & ps->extra_flags) {
    ps->cloaking = -GAME_TIME;
  }
} /* RevivePlayer */

/*
 * 
 */
static void 
DoStunned (BMPlayer *ps)
{
  switch( (ps->d_look + ps->stunned - 1)%4 + GoStop + 1) {
  case GoDown:
    SetSpriteAnime(ps->sprite,SpriteStopDown);
    break;
  case GoUp:
    SetSpriteAnime(ps->sprite,SpriteStopUp);
    break;
  case GoLeft:
    SetSpriteAnime(ps->sprite,SpriteStopLeft);
    break;
  case GoRight:
    SetSpriteAnime(ps->sprite,SpriteStopRight);
    break;
  }
  
  ps->stunned -- ;
} /* DoStunned */

/*
 *
 */
static void 
DoDie (BMPlayer *ps)
{
  if (ps->dying == DEAD_TIME) {
    SetSpriteMode (ps->sprite, SPM_MAPPED);
  }
  ps->dying --;

  if (ps->lives > 1) {
    switch (ps->d_look) {
    case GoLeft:
      SetSpriteAnime (ps->sprite, SpriteDamagedLeft);
      break;
    case GoUp:
      SetSpriteAnime (ps->sprite, SpriteDamagedUp);
      break;
    case GoRight:
      SetSpriteAnime (ps->sprite, SpriteDamagedRight);
      break;
    default:
      SetSpriteAnime (ps->sprite, SpriteDamagedDown);
      break;
    }
  } else {
    switch (ps->d_look) {
    case GoLeft:
      SetSpriteAnime (ps->sprite, SpriteDeadLeft);
      break;
    case GoUp:
      SetSpriteAnime (ps->sprite, SpriteDeadUp);
      break;
    case GoRight:
      SetSpriteAnime (ps->sprite, SpriteDeadRight);
      break;
    default:
      SetSpriteAnime (ps->sprite, SpriteDeadDown);
      break;
    }
  }
} /* DoDie */

/*
 *
 */
XBBool
CheckPlayerNear (int x, int y)
{
  int player;

  for (player = 0; player < numPlayer; player ++) {
    if ( (ABS(x*BLOCK_WIDTH - player_stat[player].x) < BLOCK_WIDTH) && 
	 (ABS(y*BLOCK_HEIGHT-BLOCK_HEIGHT - player_stat[player].y) < BLOCK_HEIGHT)) {
      return XBTrue;
    }
  }
  return XBFalse;
} /* CheckPlayerNear */

/*
 *
 */
void 
DoAllPlayers (int game_time, int *active_player)
{
  int p, player;
  
  /* if time is over, kill them all */
  if (game_time == (GAME_TIME - DEAD_TIME + 1)) {
    for (player = 0; player < numPlayer; player ++) {
      if (player_stat[player].lives>0) {
	player_stat[player].lives = 1;
	player_stat[player].dying = DEAD_TIME;
      }
    }
  }

  /* check player status */
  for (p = 0; p <numPlayer; p ++) {
    /* to permute player when drawing and stunning */
    /* quick and dirty but hopefully it solves some problems */
    player = (p + game_time) % numPlayer;
    if (player_stat[player].lives != 0) {
      
      switch(player_stat[player].dying) {
      case 0:
	/* player is alive and ... */
	if (player_stat[player].morphed) {
	  /* ... or morphed */
	  DoMorph (player_stat + player);
	} else if (player_stat[player].stunned) {
	  /* ... and stunned */
	  DoStunned (player_stat + player);
	} else {
	  /* ... walks around */
	  DoWalk (player_stat + player, game_time);
	}
	break;
	
      case 1:
	/* try to revive player */
	RevivePlayer (player_stat + player, active_player);
	break;
	
      default:
	/* player is dying */
	DoDie (player_stat + player);
	break;
      }
    }
  }
} /* DoAllPlayers */

/*
 *
 */
void
CheckPlayerHit (void)
{
  int player;
  int gridx, gridy;

  for (player = 0; player < numPlayer; player ++) {
    gridx = (player_stat[player].x + (BLOCK_WIDTH >>1))/BLOCK_WIDTH;
    gridy = (player_stat[player].y + (BLOCK_HEIGHT>>1))/BLOCK_HEIGHT + 1;
    if ( 0 != player_stat[player].lives	     && 
	 0 == player_stat[player].invincible &&
	 0 == player_stat[player].morphed    &&
	 0 == player_stat[player].dying      && 
	 CheckExplosion (gridx, gridy) ) {
      player_stat[player].dying = DEAD_TIME;
    }
  }
} /* CheckPlayerHit */

/*
 * end of file player.c
 */
