/*
 * file xblast.c - main routine
 *
 * $Id: xblast.c,v 1.33 2001/07/01 11:56:21 xblast Exp $
 *
 * Program XBLAST 
 * (C) by Oliver Vogel (e-mail: m.vogel@ndh.net)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2; or (at your option)
 * any later version
 *
 * This program is distributed in the hope that it will be entertaining,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "atom.h"
#include "cfg_main.h"
#include "demo.h"
#include "game_client.h"
#include "game_demo.h"
#include "game_local.h"
#include "game_server.h"
#include "image.h"
#include "info.h"
#include "intro.h"
#include "menu.h"
#include "random.h"
#include "socket.h"

#include "snd.h"

/*
 * going down gracefully
 */
static void
Finish (void)
{
  Dbg_Out ("Finish\n");
  /* shutdown sound */
  SND_Finish ();
  /* shutdown network */
  Socket_Finish ();
  /* clean up some modules */
  ClearShapeList ();
  ClearInfo ();
  /* save and delete current configuration */
  SaveConfig ();
  FinishConfig ();
#ifdef DEBUG_ALLOC
  Dbg_FinishAlloc ();
#endif
} /* Finish */

/*
 * main routine
 */
int
main (int argc, char *argv[])
{
  CFGSoundSetup  soundSetup;
  XBPlayerHost   hostType;

  /* init new configuration */
  InitConfig ();
  /* initialize network */
  if (! Socket_Init ()) {
    return -1;
  }
  /* Init Sound support */
  RetrieveSoundSetup (&soundSetup);
  SND_Init (&soundSetup);
  /* Initiliaze graphics engine */
  if (! GUI_Init (argc, argv) ) {
    Finish ();
    return -1;
  }
  GUI_OnQuit (Finish);
  /* init random number generator */
  SeedRandom (time (NULL));
  /* Call intro */
  DoIntro ();
  /* main loop until quit */
  while (XBPH_None != (hostType = DoMenu () ) ) {
    /* save current configurations */
    SaveConfig ();
    /* run selected game  */
    switch (hostType) {
    case XBPH_Local:  RunLocalGame ();          break;
    case XBPH_Demo:   RunDemoGame ();           break;
    case XBPH_Server: RunServerGame ();         break;
    default:          RunClientGame (hostType); break;
    }
  }
  /* close Display */
  GUI_Finish ();
  /* shutdown the rest */
  Finish ();
  /* that's all */
  return 0;
} /* main */

/*
 * end of file xblast.c
 */
